@@ -1,7 +1,7 @@
 import os
 import sys
 import codecs
-from itertools import chain
+from itertools import chain, repeat
 
 from .types import convert_type, IntRange, BOOL
 from .utils import make_str, make_default_short_help, echo
@@ -15,6 +15,10 @@ from ._compat import PY2, isidentifier
 _missing = object()
 
 
+def batch(iterable, batch_size):
+    return list(zip(*repeat(iter(iterable), batch_size)))
+
+
 def iter_params_for_processing(invocation_order, declaration_order):
     """Given a sequence of parameters in the order as should be considered
     for processing and an iterable of parameters that exist, this returns
@@ -251,124 +255,35 @@ class Context(object):
         return self.invoke(cmd, **kwargs)
 
 
-class Command(object):
-    """Commands are the basic building block of command line interfaces in
-    click.  A basic command handles command line parsing and might dispatch
-    more parsing to commands nested blow it.
+class BaseCommand(object):
+    """The base command implements the minimal API contract of commands.
+    Most code will never use this as it does not implement a lot of useful
+    functionality but it can act as the direct subclass of alternative
+    parsing methods that do not depend on the click parser.
+
+    For instance this can be used to bridge Click and other systems like
+    argparse or docopt.
+
+    Because base commands do not implement a lot of the API that other
+    parts of click take for granted they are not supported for all
+    operations.  For instance they cannot be used with the decorators
+    usually and they have no built-in callback system.
 
     :param name: the name of the command to use unless a group overrides it.
-    :param callback: the callback to invoke.  This is optional.
-    :param params: the parameters to register with this command.  This can
-                   be either :class:`Option` or :class:`Argument` objects.
-    :param help: the help string to use for this command.
-    :param epilog: like the help string but it's printed at the end of the
-                   help page after everything else.
-    :param short_help: the short help to use for this command.  This is
-                       shown on the command listing of the parent command.
-    :param add_help_option: by default each command registers a ``--help``
-                            option.  This can be disabled by this parameter.
     """
-    allow_extra_args = False
 
-    def __init__(self, name, callback=None, params=None, help=None,
-                 epilog=None, short_help=None,
-                 options_metavar='[OPTIONS]', add_help_option=True):
-        #: the name the command things it has.  Upon registering a command
+    def __init__(self, name):
+        #: the name the command thinks it has.  Upon registering a command
         #: on a :class:`Group` the group will default the command name
         #: with this information.  You should instead use the
         #: :class:`Context`\'s :attr:`~Context.info_name` attribute.
         self.name = name
-        #: the callback to execute when the command fires.  This might be
-        #: `None` in which case nothing happens.
-        self.callback = callback
-        #: the list of parameters for this command in the order they
-        #: should show up in the help page and execute.  Eager parameters
-        #: will automatically be handled before non eager ones.
-        self.params = params or []
-        self.help = help
-        self.epilog = epilog
-        self.options_metavar = options_metavar
-        if short_help is None and help:
-            short_help = make_default_short_help(help)
-        self.short_help = short_help
-        if add_help_option:
-            self.add_help_option()
-
-    def add_help_option(self):
-        """Adds a help option to the command."""
-        help_option()(self)
-
-    def make_parser(self, ctx):
-        """Creates the underlying option parser for this command."""
-        parser = OptionParser(ctx)
-        for param in self.params:
-            param.add_to_parser(parser, ctx)
-        return parser
 
     def get_usage(self, ctx):
-        """Formats the usage line into a string and returns it.  This
-        creates a formatter and will call into the following formatting
-        methods:
-
-        -   :meth:`format_usage`
-        """
-        formatter = ctx.make_formatter()
-        self.format_usage(ctx, formatter)
-        return formatter.getvalue().rstrip('\n')
+        raise NotImplementedError('Base commands cannot get usage')
 
     def get_help(self, ctx):
-        """Formats the help into a string and returns it.  This creates a
-        formatter and will call into the following formatting methods:
-
-        -   :meth:`format_usage`
-        -   :meth:`format_help`
-        -   :meth:`format_options`
-        -   :meth:`format_epilog`
-        """
-        formatter = ctx.make_formatter()
-        self.format_usage(ctx, formatter)
-        self.format_help(ctx, formatter)
-        self.format_options(ctx, formatter)
-        self.format_epilog(ctx, formatter)
-        return formatter.getvalue().rstrip('\n')
-
-    def format_usage(self, ctx, formatter):
-        """Writes the usage line into the formatter."""
-        pieces = self.collect_usage_pieces(ctx)
-        formatter.write_usage(ctx.command_path, ' '.join(pieces))
-
-    def format_help(self, ctx, formatter):
-        """Writes the help into the formatter if it exists."""
-        if self.help:
-            formatter.write_paragraph()
-            with formatter.indentation():
-                formatter.write_text(self.help)
-
-    def format_options(self, ctx, formatter):
-        """Writes all the options into the formatter if they exist."""
-        opts = []
-        for param in self.params:
-            rv = param.get_help_record(ctx)
-            if rv is not None:
-                opts.append(rv)
-
-        if opts:
-            with formatter.section('Options'):
-                formatter.write_dl(opts)
-
-    def format_epilog(self, ctx, formatter):
-        """Writes the epilog into the formatter if it exists."""
-        if self.epilog:
-            formatter.write_paragraph()
-            with formatter.indentation():
-                formatter.write_text(self.epilog)
-
-    def collect_usage_pieces(self, ctx):
-        """Returns all the pieces that go into the usage line."""
-        rv = [self.options_metavar]
-        for param in self.params:
-            rv.extend(param.get_usage_pieces(ctx))
-        return rv
+        raise NotImplementedError('Base commands cannot get help')
 
     def make_context(self, info_name, args, parent=None, **extra):
         """This function when given an info name and arguments will kick
@@ -377,7 +292,7 @@ class Command(object):
 
         :param info_name: the info name for this invokation.  Generally this
                           is the most descriptive name for the script or
-                          command.  For the toplevel script is is usually
+                          command.  For the toplevel script it's usually
                           the name of the script, for commands below it it's
                           the name of the script.
         :param args: the arguments to parse as list of strings.
@@ -399,30 +314,19 @@ class Command(object):
         and parses the arguments, then modifies the context as necessary.
         This is automatically invoked by :meth:`make_context`.
         """
-        parser = self.make_parser(ctx)
-        opts, args, param_order = parser.parse_args(args=args)
-
-        for param in iter_params_for_processing(param_order, self.params):
-            value, args = param.handle_parse_result(ctx, opts, args)
-
-        if args and not self.allow_extra_args:
-            ctx.fail('Got unexpected extra argument%s (%s)'
-                     % (len(args) != 1 and 's' or '',
-                        ' '.join(map(make_str, args))))
-
-        ctx.args = args
+        raise NotImplementedError('Base commands do not know how to parse '
+                                  'arguments.')
 
     def invoke(self, ctx):
-        """Given a context, this invokes the attached callback (if it exists)
-        in the right way.
+        """Given a context, this invokes the command.  The default
+        implementation is raising a not implemented error.
         """
-        if self.callback is not None:
-            ctx.invoke(self.callback, **ctx.params)
+        raise NotImplementedError('Base commands are not invokable by default')
 
     def main(self, args=None, prog_name=None, **extra):
         """This is the way to invoke a script with all the bells and
-        whistles as command line application.  This will always terminate
-        the application after calling.  If this is not wanted, ``SystemExit``
+        whistles as a command line application.  This will always terminate
+        the application after a call.  If this is not wanted, ``SystemExit``
         needs to be caught.
 
         This method is also available by directly calling the instance of
@@ -431,7 +335,7 @@ class Command(object):
         :param args: the arguments that should be used for parsing.  If not
                      provided, ``sys.argv[1:]`` is used.
         :param prog_name: the program name that should be used.  By default
-                          the progam name is constructed by taking the file
+                          the program name is constructed by taking the file
                           name from ``sys.argv[0]``.
         :param extra: extra keyword arguments are forwarded to the context
                       constructor.  See :class:`Context` for more information.
@@ -480,6 +384,146 @@ class Command(object):
         return self.main(*args, **kwargs)
 
 
+class Command(BaseCommand):
+    """Commands are the basic building block of command line interfaces in
+    click.  A basic command handles command line parsing and might dispatch
+    more parsing to commands nested below it.
+
+    :param name: the name of the command to use unless a group overrides it.
+    :param callback: the callback to invoke.  This is optional.
+    :param params: the parameters to register with this command.  This can
+                   be either :class:`Option` or :class:`Argument` objects.
+    :param help: the help string to use for this command.
+    :param epilog: like the help string but it's printed at the end of the
+                   help page after everything else.
+    :param short_help: the short help to use for this command.  This is
+                       shown on the command listing of the parent command.
+    :param add_help_option: by default each command registers a ``--help``
+                            option.  This can be disabled by this parameter.
+    """
+    allow_extra_args = False
+
+    def __init__(self, name, callback=None, params=None, help=None,
+                 epilog=None, short_help=None,
+                 options_metavar='[OPTIONS]', add_help_option=True):
+        BaseCommand.__init__(self, name)
+        #: the callback to execute when the command fires.  This might be
+        #: `None` in which case nothing happens.
+        self.callback = callback
+        #: the list of parameters for this command in the order they
+        #: should show up in the help page and execute.  Eager parameters
+        #: will automatically be handled before non eager ones.
+        self.params = params or []
+        self.help = help
+        self.epilog = epilog
+        self.options_metavar = options_metavar
+        if short_help is None and help:
+            short_help = make_default_short_help(help)
+        self.short_help = short_help
+        if add_help_option:
+            self.add_help_option()
+
+    def add_help_option(self):
+        """Adds a help option to the command."""
+        help_option()(self)
+
+    def get_usage(self, ctx):
+        formatter = ctx.make_formatter()
+        self.format_usage(ctx, formatter)
+        return formatter.getvalue().rstrip('\n')
+
+    def format_usage(self, ctx, formatter):
+        """Writes the usage line into the formatter."""
+        pieces = self.collect_usage_pieces(ctx)
+        formatter.write_usage(ctx.command_path, ' '.join(pieces))
+
+    def collect_usage_pieces(self, ctx):
+        """Returns all the pieces that go into the usage line and returns
+        it as a list of strings.
+        """
+        rv = [self.options_metavar]
+        for param in self.params:
+            rv.extend(param.get_usage_pieces(ctx))
+        return rv
+
+    def make_parser(self, ctx):
+        """Creates the underlying option parser for this command."""
+        parser = OptionParser(ctx)
+        for param in self.params:
+            param.add_to_parser(parser, ctx)
+        return parser
+
+    def get_help(self, ctx):
+        """Formats the help into a string and returns it.  This creates a
+        formatter and will call into the following formatting methods:
+        """
+        formatter = ctx.make_formatter()
+        self.format_help(ctx, formatter)
+        return formatter.getvalue().rstrip('\n')
+
+    def format_help(self, ctx, formatter):
+        """Writes the help into the formatter if it exists.
+
+        This calls into the following methods:
+
+        -   :meth:`format_usage`
+        -   :meth:`format_help_text`
+        -   :meth:`format_options`
+        -   :meth:`format_epilog`
+        """
+        self.format_usage(ctx, formatter)
+        self.format_help_text(ctx, formatter)
+        self.format_options(ctx, formatter)
+        self.format_epilog(ctx, formatter)
+
+    def format_help_text(self, ctx, formatter):
+        """Writes the help text to the formatter if it exists."""
+        if self.help:
+            formatter.write_paragraph()
+            with formatter.indentation():
+                formatter.write_text(self.help)
+
+    def format_options(self, ctx, formatter):
+        """Writes all the options into the formatter if they exist."""
+        opts = []
+        for param in self.params:
+            rv = param.get_help_record(ctx)
+            if rv is not None:
+                opts.append(rv)
+
+        if opts:
+            with formatter.section('Options'):
+                formatter.write_dl(opts)
+
+    def format_epilog(self, ctx, formatter):
+        """Writes the epilog into the formatter if it exists."""
+        if self.epilog:
+            formatter.write_paragraph()
+            with formatter.indentation():
+                formatter.write_text(self.epilog)
+
+    def parse_args(self, ctx, args):
+        parser = self.make_parser(ctx)
+        opts, args, param_order = parser.parse_args(args=args)
+
+        for param in iter_params_for_processing(param_order, self.params):
+            value, args = param.handle_parse_result(ctx, opts, args)
+
+        if args and not self.allow_extra_args:
+            ctx.fail('Got unexpected extra argument%s (%s)'
+                     % (len(args) != 1 and 's' or '',
+                        ' '.join(map(make_str, args))))
+
+        ctx.args = args
+
+    def invoke(self, ctx):
+        """Given a context, this invokes the attached callback (if it exists)
+        in the right way.
+        """
+        if self.callback is not None:
+            ctx.invoke(self.callback, **ctx.params)
+
+
 class MultiCommand(Command):
     """A multi command is the basic implementation of a command that
     dispatches to subcommands.  The most common version is the
@@ -684,7 +728,7 @@ class Parameter(object):
                         argument.  This is a list of flags or argument
                         names.
     :param type: the type that should be used.  Either a :class:`ParamType`
-                 or a python type.  The latter is converted into the former
+                 or a python type.  The later is converted into the former
                  automatically if supported.
     :param required: controls if this is optional or not.
     :param default: the default value if omitted.  This can also be a callable
@@ -788,7 +832,7 @@ class Parameter(object):
                 self.opts, self.secondary_opts)),
         )
 
-    def value_from_envvar(self, ctx):
+    def resolve_envvar_value(self, ctx):
         if self.envvar is None:
             return
         if isinstance(self.envvar, (tuple, list)):
@@ -799,6 +843,12 @@ class Parameter(object):
         else:
             return os.environ.get(self.envvar)
 
+    def value_from_envvar(self, ctx):
+        rv = self.resolve_envvar_value(ctx)
+        if rv is not None and self.nargs != 1:
+            rv = self.type.split_envvar_value(rv)
+        return rv
+
     def handle_parse_result(self, ctx, opts, args):
         value = self.consume_value(ctx, opts)
         value = self.full_process_value(ctx, value)
@@ -836,11 +886,11 @@ class Option(Parameter):
     :param flag_value: which value should be used for this flag if it's
                        enabled.  This is set to a boolean automatically if
                        the option string contains a slash to mark two options.
-    :param multiple: if this is set to True then the argument is accepted
-                     multiple times and recorded.  This is similar to nargs
+    :param multiple: if this is set to `True` then the argument is accepted
+                     multiple times and recorded.  This is similar to ``nargs``
                      in how it works but supports arbitrary number of
                      arguments.
-    :param count: this flag makes an option increment an integer
+    :param count: this flag makes an option increment an integer.
     :param allow_from_autoenv: if this is enabled then the value of this
                                parameter will be pulled from an environment
                                variable in case a prefix is defined on the
@@ -1038,14 +1088,26 @@ class Option(Parameter):
                       confirmation_prompt=self.confirmation_prompt,
                       value_proc=lambda x: self.process_value(ctx, x))
 
-    def value_from_envvar(self, ctx):
-        rv = Parameter.value_from_envvar(self, ctx)
+    def resolve_envvar_value(self, ctx):
+        rv = Parameter.resolve_envvar_value(self, ctx)
         if rv is not None:
             return rv
-        if self.allow_from_autoenv and ctx.auto_envvar_prefix is not None:
+        if self.allow_from_autoenv and \
+           ctx.auto_envvar_prefix is not None:
             envvar = '%s_%s' % (ctx.auto_envvar_prefix, self.name.upper())
             return os.environ.get(envvar)
 
+    def value_from_envvar(self, ctx):
+        rv = self.resolve_envvar_value(ctx)
+        if rv is None:
+            return None
+        value_depth = (self.nargs != 1) + bool(self.multiple)
+        if value_depth > 0 and rv is not None:
+            rv = self.type.split_envvar_value(rv)
+            if self.multiple and self.nargs != 1:
+                rv = batch(rv, self.nargs)
+        return rv
+
     def full_process_value(self, ctx, value):
         if value is None and self.prompt is not None:
             return self.prompt_for_value(ctx)
