@@ -87,7 +87,7 @@ def _make_command(f, name, attrs, cls):
 def command(name=None, cls=None, **attrs):
     """Creates a new :class:`Command` and uses the decorated function as
     callback.  This will also automatically attach all decorated
-    :func:`option`\s and :func:`argument`\s as paramters to the command.
+    :func:`option`\s and :func:`argument`\s as parameters to the command.
 
     The name of the command defaults to the name of the function.  If you
     want to change that, you can pass the intended name as the first
